package com.edmz.common.model;

import java.awt.Font;
import java.util.Iterator;
import java.util.Vector;

import javax.swing.JLabel;
import javax.swing.table.AbstractTableModel;

public class EventDataModel extends AbstractTableModel {

	private static final long serialVersionUID = 1L;
	
	public final static int COLINDEX_FORMATTED_TIME = 0;
    public final static int COLINDEX_EVENT_TEXT = 1;
    private String COL_FORMATTED_TIME = null;
    private String COL_EVENT_TEXT = null;
    protected String[] columnNames = null;
    protected int[] columnSize = null;
    protected Class<?> columnClasses[] = null;

    /**
     * This Vector contains all EventData.
     */
    private Vector<EventData> rows = new Vector<EventData>();
    

    public Vector<EventData> getRows() {
		return rows;
	}

	public EventDataModel() {
        // Property initialization is done ... for each property. Nothing to do
        // here.
        super();
        
        // Initialization for column name, type and size.
        this.COL_FORMATTED_TIME = "Event Time";
        this.COL_EVENT_TEXT = "Event Text";

        this.columnNames = new String[] {
                this.COL_FORMATTED_TIME, this.COL_EVENT_TEXT
        };

        this.columnSize = new int[] {
                25, 325
        };
        
        this.columnClasses = new Class[] {
                String.class, String.class
        };
    }
    
    public void addEvent(int eventInfo, int ms, String eventText)
    throws Exception {
		synchronized (this.rows) {
	        EventData ed = new EventData(eventInfo, ms, eventText); 
	        if (ed != null) {
	            // The file is Ok, let's add it.
	            this.rows.add(ed);
	            fireTableDataChanged();
	        }
		}
    }

    public int highLightRow(long ms) {
        int row = 0;

        synchronized (this.rows) {
            Iterator<EventData> i = this.rows.iterator();
            while (i.hasNext()) {
            	row += 1;
//                if (ms <= (i.next().getMs())) {
                if (ms <= (i.next().getMs() - 1000)) {
                    break;
                }
            }
        }// synchronized

        return row;
    }

    /**
     * Ask for the file contained at specified row number.
     * 
     * @param row The row number
     * @return The return instance of File.
     */
    public EventData getEventDataAt(int row) {
        int size = -1;
        if (row >= 0) {
            try {
                synchronized (this.rows) {
                    size = this.rows.size();
                    return (row < size) ? this.rows.get(row) : null;
                }
            } catch (ArrayIndexOutOfBoundsException e) {
            }
        }
        return null;
    }
    
    /** @see javax.swing.table.TableModel#getColumnCount() */
    public int getColumnCount() {
        return this.columnNames.length;
    }

    /** @see javax.swing.table.TableModel#getRowCount() */
    public int getRowCount() {
        synchronized (this.rows) {
            return this.rows.size();
        }
    }

    /**
     * Always return false here : no editable cell.
     * 
     * @see javax.swing.table.TableModel#isCellEditable(int, int)
     */
    @Override
    public boolean isCellEditable(int arg0, int arg1) {
        // No editable columns.
        return false;
    }

    /**
     * Return true if this column can be sorted.
     * 
     * @param col The index of the column which can sortable or not.
     * @return true if the column can be sorted. false otherwise.
     */
    public boolean isSortable(int col) {
        return (Boolean.class != getColumnClass(col));
    }

    /**
     * @see javax.swing.table.TableModel#getColumnClass(int)
     */
    @SuppressWarnings("unchecked")
    @Override
    public Class getColumnClass(int arg0) {
        return this.columnClasses[arg0];
    }

    /**
     * This method doesn't do anything : no changeable values.
     * 
     * @see javax.swing.table.TableModel#setValueAt(java.lang.Object, int, int)
     */
    @Override
    public void setValueAt(Object arg0, int arg1, int arg2) {
//        this.uploadPolicy.displayWarn(this.getClass().getName()
//                + ".setValueAt: no action");
    }

    /**
     * @see javax.swing.table.TableModel#getColumnName(int)
     */
    @Override
    public String getColumnName(int arg0) {
        return this.columnNames[arg0];
    }

    /**
     * Retrieves the default colum size of a column.
     * 
     * @param col The index of the column to query.
     * @return the default size of the requested column.
     */
    public int getColumnSize(int col) {
        return this.columnSize[col];
    }


	@Override
	public Object getValueAt(int rowIndex, int columnIndex) {
        EventData eventData = getEventDataAt(rowIndex);
        if (eventData != null) {
            String colName = getColumnName(columnIndex);
            // Don't know if it will be useful, but the switch below allows the
            // column to be in any order.            
            if (colName.equals(this.COL_FORMATTED_TIME)) {
                return eventData.getFormattedTime();
            } else if (colName.equals(this.COL_EVENT_TEXT)) {
                return eventData.getEventText();
            } 
        }
        return null;
	}
}
