package com.edmz.common.ui;

import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.ClipboardOwner;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.FlavorEvent;
import java.awt.datatransfer.FlavorListener;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.io.IOException;
import java.util.Timer;
import java.util.TimerTask;

public class MacOSClipboard extends Clipboard {

	static private Clipboard m_SystemClipboard;

	static private MacOSClipboard m_MacOSClipboard;

	private static DataFlavor[] m_Flavors;

	static public Clipboard getClipboard() {
		if (m_MacOSClipboard == null)
			m_MacOSClipboard = new MacOSClipboard();
		return m_MacOSClipboard;
	}

	private MacOSClipboard() {
		super(Toolkit.getDefaultToolkit().getSystemClipboard().getName());
		m_SystemClipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
		m_MacOSClipboard = this;

	}


	public synchronized void addFlavorListener(FlavorListener listener) {
		// TODO Auto-generated method stub
		m_SystemClipboard.addFlavorListener(listener);
	}


	public DataFlavor[] getAvailableDataFlavors() {
		// TODO Auto-generated method stub
		return m_SystemClipboard.getAvailableDataFlavors();
	}


	public synchronized Transferable getContents(Object requestor) {
		// TODO Auto-generated method stub
		return m_SystemClipboard.getContents(requestor);
	}


	public Object getData(DataFlavor flavor) throws UnsupportedFlavorException,
			IOException {
		return m_SystemClipboard.getData(flavor);
	}


	public boolean isDataFlavorAvailable(DataFlavor flavor) {
		return m_SystemClipboard.isDataFlavorAvailable(flavor);
	}


	public synchronized void removeFlavorListener(FlavorListener listener) {
		m_SystemClipboard.removeFlavorListener(listener);
	}

	public synchronized FlavorListener[] getFlavorListeners() {
		return m_SystemClipboard.getFlavorListeners();
	}


	public String getName() {
		return m_SystemClipboard.getName();
	}


	public synchronized void setContents(Transferable contents,
			ClipboardOwner owner) {
		final ClipboardOwner realOwner = owner;
		m_SystemClipboard.setContents(contents, new ClipboardOwner() {
			public void lostOwnership(Clipboard clipboard, Transferable contents) {
				realOwner.lostOwnership(clipboard, contents);
				fireFlavorsChanged();
			}
		});
		fireFlavorsChanged();
	}

	static/* packate */void fireFlavorsChanged() {
//		for (FlavorListener listener : m_SystemClipboard.getFlavorListeners())
//			listener.flavorsChanged(new FlavorEvent(m_SystemClipboard));
		FlavorListener[] listeners = m_SystemClipboard.getFlavorListeners();
		FlavorListener listener = null;
		for (int i = 0; i < m_SystemClipboard.getFlavorListeners().length; i++) {
			listener = listeners[i];
			listener.flavorsChanged(new FlavorEvent(m_SystemClipboard));
		}
		
		m_Flavors = m_SystemClipboard.getAvailableDataFlavors();
	}

	/* package */boolean isFlavorsChanged() {
		DataFlavor[] current = m_SystemClipboard.getAvailableDataFlavors();
		boolean changed = false;
		if (m_Flavors != null && current.length == m_Flavors.length) {
			for (int i = 0; i < m_Flavors.length; ++i)
				if (!current[i].equals(m_Flavors[i]))
					return true;
			m_Flavors = current;
		} else
			changed = true;
		return true;
	}

	/* package */void startClipboardDaemon(int period) {

		Timer daemon = new Timer(true);
		daemon.schedule(new TimerTask() {
			public void run() {
				javax.swing.SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						if (isFlavorsChanged())
							fireFlavorsChanged();
					}
				});
			}
		}, period, period);
	}

}
