/*
 *  SSHTools - Java SSH2 API
 *
 *  Copyright (C) 2002 Lee David Painter.
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public License
 *  as published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *
 *  You may also distribute it and/or modify it under the terms of the
 *  Apache style J2SSH Software License. A copy of which should have
 *  been provided with the distribution.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  License document supplied with your distribution for more details.
 *
 */

package com.sshtools.common.ui;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.ClipboardOwner;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.FlavorEvent;
import java.awt.datatransfer.FlavorListener;
import java.awt.datatransfer.StringSelection;
import java.awt.datatransfer.Transferable;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.lang.reflect.Field;

import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JMenuBar;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JSlider;
import javax.swing.JToggleButton;
import javax.swing.Timer;
import javax.swing.UIManager;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.apache.commons.lang.time.DurationFormatUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.edmz.common.ui.EGPPlaybackMenuBar;
import com.sshtools.sshvnc.SshVNCViewer;

/**
 *
 *
 * @author $author$
 * @version $Revision: 1.14 $
 */

/*
 * 3/14/11 Bob Baske - Custom modifications to handle playback and speed controls
 */
public class StatusBar
    extends JPanel implements ActionListener, ClipboardOwner {
    /**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	private static Log log = LogFactory.getLog(StatusBar.class);
    
//    private static final int FixedWidth = 120;
    private static final int FixedWidth = 60;
    
    private static int TIMER_INTERVAL = 500;
   
    public static String OPENED = "Channel opened";
    public static String EOF = "Channel end of file";
    public static String CLOSED = "Channel closed";
  
  private SshVNCViewer viewer = null;
  /**  */
  public final static Icon GREEN_LED_ON = new ResourceIcon(StatusBar.class,
      "/resources/greenledon.png");

  /**  */
  public final static Icon GREEN_LED_OFF = new ResourceIcon(StatusBar.class,
      "/resources/greenledoff.png");

  /**  */
  public final static Icon RED_LED_ON = new ResourceIcon(StatusBar.class,
      "/resources/redledon.png");

  /**  */
  public final static Icon RED_LED_OFF = new ResourceIcon(StatusBar.class,
      "/resources/redledoff.png");
  
  public final static Icon CAD_ICON = new ResourceIcon(StatusBar.class,
  "/resources/cad.png");
  
  public final static Icon CTRLESC_ICON = new ResourceIcon(StatusBar.class,
  "/resources/ctrlesc.png");
    
  public final static Icon CONN_ON_ICON = new ResourceIcon(StatusBar.class,
  "/resources/connGreenledOn.png");
  
  public final static Icon CONN_OFF_ICON = new ResourceIcon(StatusBar.class,
  "/resources/connGreenledOff.png");
  
  public final static Icon CONN_STOP_ICON = new ResourceIcon(StatusBar.class,
  "/resources/connRedledOn.png");
  
  public final static Icon SEND_ON_ICON = new ResourceIcon(StatusBar.class,
  "/resources/sendGreenledOn.png");
  
  public final static Icon SEND_OFF_ICON = new ResourceIcon(StatusBar.class,
  "/resources/sendGreenledOff.png");
  
  public final static Icon SEND_STOP_ICON = new ResourceIcon(StatusBar.class,
  "/resources/sendRedledOn.png");
  
  public final static Icon RECV_ON_ICON = new ResourceIcon(StatusBar.class,
  "/resources/recvGreenledOn.png");
  
  public final static Icon RECV_OFF_ICON = new ResourceIcon(StatusBar.class,
  "/resources/recvGreenledOff.png");
  
  public final static Icon RECV_STOP_ICON = new ResourceIcon(StatusBar.class,
  "/resources/recvRedledOn.png");
  
  
  public final static Icon REFRESH_ICON = new ResourceIcon(StatusBar.class,
  "/resources/e_refresh.gif");

  public final static String TIME_FORMAT = "H:mm:ss";
  
  public final static String CDF = "currentDataFlavors";
  
  /* Control action constants */
  public final static String PAUSE = "pause";
  public final static String LOOP = "loop";
  public final static String HALF_SPEED = "half";
  public final static String NORMAL_SPEED = "normal";
  public final static String TWO_TIMES_SPEED = "twoTimes";
  public final static String FOUR_TIMES_SPEED = "fourTimes";
  public final static String EIGHT_TIMES_SPEED = "eightTimes";
  public final static String SIXTEEN_TIMES_SPEED = "sixteenTimes";
  public final static String TAB = "tab";
  public final static String SHIFT_TAB = "shiftTab";
  public final static String CAD = "cad";
  public final static String CTRLESC = "ctrlesc";
  public final static String REFRESH = "refresh";
  public final static String OPEN_EVENTS = "events";
  public final static String OPEN_KEYSTROKES = "keystrokes";
//  public final static String COPY_TO = "copyTo"; 
  
  private boolean block = false;
  private boolean playback = false;
  private boolean allowClipboard = true;
  private boolean allowCAD = true;
  //
  private StatusLabel statusText;
  private StatusLabel host;
  private StatusLabel user;
//  private StatusLabel rid;
  private JLabel sending;
  private JLabel receiving;
  private Timer sendingTimer;
  private Timer receivingTimer;
  private ReceiveTimerAction rta = new ReceiveTimerAction();
  private SendTimerAction sta = new SendTimerAction();
  
  /******************** playback controls ************************/
  private JSlider pbar;
  private JLabel counter;
  private JLabel conn;

private JLabel countTo;
//  private Box speedControls;
  private JToggleButton pause; 
  private JToggleButton loopButton;
  
  private JButton cad;
  private JButton ctrlesc;
  private JButton refresh;
  
//  private JLabel pausedInd;
  private Timer pauseTimer;
  private int pausedValue;
  private boolean paused = false;
  private boolean loop = false;
  private int duration;
  private Timer countTimer;
  private long now = 0;
  private boolean started = false;
  
  private JRadioButton half;
  private JRadioButton normal;
  private JRadioButton twoTimes;
  private JRadioButton fourTimes;
  private JRadioButton eightTimes;
  private JRadioButton sixteenTimes;
  
  private JMenuBar menuBar = null;
  
  CBFlavorListener cfl = null;

  private ButtonGroup group;
  /* .5 = 1/2 speed, 1 = normal, 2 = 2x, 4 = 4x, 
   * 8 = 8x, 16 = 16x normal speed 
   */
  double speed = 1; 
  
  boolean sliderAction = false;
  
/******************** playback controls ************************/
  /**
   * @param layout
   * @param isDoubleBuffered
   * @param viewer
   */
  public StatusBar(SshVNCViewer viewer) {
      super(new GridBagLayout());
      this.viewer = viewer;
      this.init();   
  }  
  
  
  /**
   * Creates a new StatusBar object.
   */
  public StatusBar() {
    super(new GridBagLayout());
    this.init();
  }
  
  private void init() {
      UIManager.put("SliderUI", "com.edmz.common.ui.EdmzSliderUI");
      GridBagConstraints gbc = new GridBagConstraints();
      gbc.anchor = GridBagConstraints.WEST;
      gbc.fill = GridBagConstraints.BOTH;
           
      gbc.weightx = 0.1;
      host = new StatusLabel();
      host.setToolTipText("Displays System name.");
      UIUtil.jGridBagAdd(this, host, gbc, 1);

      user = new StatusLabel();
      user.setToolTipText("Displays Account name.");
      UIUtil.jGridBagAdd(this, user, gbc, 1);
//      rid = new StatusLabel();
//      UIUtil.jGridBagAdd(this, rid, gbc, 1);
//      rid.setVisible(false);
      /******************** record - cntl+alt+del, refresh ************************/      
      cad = new JButton("", CAD_ICON);
      cad.setFont(getFont().deriveFont(10f));
      cad.setToolTipText("Click to send CTRL+ALT+DEL.");
      cad.setActionCommand(StatusBar.CAD);
      cad.addActionListener(this);
      cad.setBorder(BorderFactory.createRaisedBevelBorder());
      cad.setFocusable(true);
      cad.setEnabled(true); 
      cad.setVisible(false);
      gbc.weightx = 0.0;
      UIUtil.jGridBagAdd(this, cad, gbc, 1);
      
      ctrlesc = new JButton("", CTRLESC_ICON);
      ctrlesc.setFont(getFont().deriveFont(10f));
      ctrlesc.setToolTipText("Click to send CTRL+ESC.");
      ctrlesc.setActionCommand(StatusBar.CTRLESC);
      ctrlesc.addActionListener(this);
      ctrlesc.setBorder(BorderFactory.createRaisedBevelBorder());
      ctrlesc.setFocusable(true);
      ctrlesc.setEnabled(true); 
      ctrlesc.setVisible(false);
      gbc.weightx = 0.0;
      UIUtil.jGridBagAdd(this, ctrlesc, gbc, 1);
      
      refresh = new JButton("", REFRESH_ICON);
      refresh.setFont(getFont().deriveFont(10f));
      refresh.setToolTipText("Click to Refresh.");
      refresh.setActionCommand(StatusBar.REFRESH);
      refresh.addActionListener(this);
      refresh.setBorder(BorderFactory.createRaisedBevelBorder());
      refresh.setFocusable(true);
      refresh.setEnabled(true); 
      refresh.setVisible(false);
      gbc.weightx = 0.0;
      UIUtil.jGridBagAdd(this, refresh, gbc, 1);
            
      /******************** record - cntl+alt+del, refresh ************************/  
      /******************** playback controls ************************/ 
      //  Progress bar starts here   
      pbar = new JSlider();
      pbar.setToolTipText("Press pause to reposition the slider.");
      pbar.setEnabled(false);
      pbar.setVisible(false);
      pbar.setBorder(BorderFactory.createLoweredBevelBorder());
      pbar.addChangeListener(new SliderAjustmentListener());

      gbc.weightx = 2.0;

      UIUtil.jGridBagAdd(this, pbar, gbc, 1);
      // Counter (H:mm:ss)/(H:mm:ss) starts here
      counter = new JLabel();
      counter.setVisible(false);

      counter.setBorder(BorderFactory.createCompoundBorder(
            BorderFactory.createLoweredBevelBorder(),
            BorderFactory.createEmptyBorder(1, 1, 1, 1)));
      counter.setHorizontalAlignment(JLabel.RIGHT);
      counter.setFont(getFont().deriveFont(Font.BOLD, 11f));

      countTo = new JLabel();
      countTo.setVisible(false);
      countTo.setBorder(BorderFactory.createCompoundBorder(
              BorderFactory.createLoweredBevelBorder(),
              BorderFactory.createEmptyBorder(1, 1, 1, 1)));
      countTo.setHorizontalAlignment(JLabel.LEFT);
      countTo.setFont(getFont().deriveFont(Font.BOLD, 11f));
 
      gbc.weightx = 0.0;
 
      UIUtil.jGridBagAdd(this, counter, gbc, 1);  
      UIUtil.jGridBagAdd(this, countTo, gbc, 1);  
      
      // Pause button 
      pauseTimer = new Timer(TIMER_INTERVAL, new TimerAction());
      pause = new JToggleButton("Pause", GREEN_LED_ON, false);
      pause.setFont(getFont().deriveFont(10f));
      pause.setToolTipText("Click to pause session playback.");
      pause.setActionCommand(StatusBar.PAUSE);
      pause.addActionListener(this);
      pause.setBorder(BorderFactory.createRaisedBevelBorder());
      pause.setFocusable(true);
      pause.setEnabled(false);
      pause.setVisible(false);
      gbc.weightx = 0.0;

      UIUtil.jGridBagAdd(this, pause, gbc, 1);
      
//    Loop button
      if (isLoop()) {
    	  loopButton = new JToggleButton("Loop", GREEN_LED_ON, false);
    	  loopButton.setBorder(BorderFactory.createLoweredBevelBorder());
      } else {
    	  loopButton = new JToggleButton("Loop", RED_LED_ON, false);
    	  loopButton.setBorder(BorderFactory.createRaisedBevelBorder());
      }
      loopButton.setFont(getFont().deriveFont(10f));
      loopButton.setToolTipText("Click to loop session playback.");
      loopButton.setActionCommand(StatusBar.LOOP);
      loopButton.addActionListener(this);
      loopButton.setFocusable(true);
      loopButton.setEnabled(false); 
      loopButton.setVisible(false);
      gbc.weightx = 0.0;

      UIUtil.jGridBagAdd(this, loopButton, gbc, 1);
      //Speed controls start here

      half = new JRadioButton(".5x", false);  
      half.setToolTipText("Click to change playback speed.");
      half.setActionCommand(StatusBar.HALF_SPEED);
      half.setFont(getFont().deriveFont(9f));
      half.setBorder(BorderFactory.createCompoundBorder(
              BorderFactory.createLoweredBevelBorder(),
              BorderFactory.createEmptyBorder(1, 1, 1, 1)));
      half.setIconTextGap(0);
      half.addActionListener(this);
      half.setEnabled(false);
      half.setVisible(false);
      normal = new JRadioButton("1x", true);
      normal.setToolTipText("Click to change playback speed.");
      normal.setActionCommand(StatusBar.NORMAL_SPEED);
      normal.setFont(getFont().deriveFont(9f));
      normal.setBorder(BorderFactory.createCompoundBorder(
              BorderFactory.createLoweredBevelBorder(),
              BorderFactory.createEmptyBorder(1, 1, 1, -2)));
      normal.setIconTextGap(1);
      normal.addActionListener(this);
      normal.setEnabled(false);
      normal.setVisible(false);
      twoTimes = new JRadioButton("2x", false);
      twoTimes.setToolTipText("Click to change playback speed.");
      twoTimes.setActionCommand(StatusBar.TWO_TIMES_SPEED);
      twoTimes.setFont(getFont().deriveFont(7f));
      twoTimes.setBorder(BorderFactory.createCompoundBorder(
              BorderFactory.createLoweredBevelBorder(),
              BorderFactory.createEmptyBorder(1, 1, 1, -2)));
      twoTimes.setIconTextGap(1);
      twoTimes.addActionListener(this);
      twoTimes.setEnabled(false);
      twoTimes.setVisible(false);
      fourTimes = new JRadioButton("4x", false);
      fourTimes.setToolTipText("Click to change playback speed.");
      fourTimes.setActionCommand(StatusBar.FOUR_TIMES_SPEED);
      fourTimes.setFont(getFont().deriveFont(9f));
      fourTimes.setBorder(BorderFactory.createCompoundBorder(
              BorderFactory.createLoweredBevelBorder(),
              BorderFactory.createEmptyBorder(1, 1, 1, -2)));
      fourTimes.setIconTextGap(1);
      fourTimes.addActionListener(this);
      fourTimes.setEnabled(false);
      fourTimes.setVisible(false);
      eightTimes = new JRadioButton("8x", false);
      eightTimes.setToolTipText("Click to change playback speed.");
      eightTimes.setActionCommand(StatusBar.EIGHT_TIMES_SPEED);
      eightTimes.setFont(getFont().deriveFont(9f));
      eightTimes.setBorder(BorderFactory.createCompoundBorder(
              BorderFactory.createLoweredBevelBorder(),
              BorderFactory.createEmptyBorder(1, 1, 1, -2)));
      eightTimes.setIconTextGap(1);
      eightTimes.addActionListener(this);
      eightTimes.setEnabled(false);
      eightTimes.setVisible(false);
      sixteenTimes = new JRadioButton("16x", false);
      sixteenTimes.setToolTipText("Click to change playback speed.");
      sixteenTimes.setActionCommand(StatusBar.SIXTEEN_TIMES_SPEED);
      sixteenTimes.setFont(getFont().deriveFont(9f));
      sixteenTimes.setBorder(BorderFactory.createCompoundBorder(
              BorderFactory.createLoweredBevelBorder(),
              BorderFactory.createEmptyBorder(1, 1, 1, -2)));
      sixteenTimes.setIconTextGap(1);
      sixteenTimes.addActionListener(this);
      sixteenTimes.setEnabled(false);
      sixteenTimes.setVisible(false);
      
      group = new ButtonGroup();
      group.add(half);
      group.add(normal);
      group.add(twoTimes);
      group.add(fourTimes);
      group.add(eightTimes);
      group.add(sixteenTimes);

      gbc.weightx = 0.1;
      gbc.weighty = 0.1;
      
      UIUtil.jGridBagAdd(this, half, gbc, 1);
      UIUtil.jGridBagAdd(this, normal, gbc, 1);
      UIUtil.jGridBagAdd(this, twoTimes, gbc, 1);
      UIUtil.jGridBagAdd(this, fourTimes, gbc, 1);
      UIUtil.jGridBagAdd(this, eightTimes, gbc, 1);
      UIUtil.jGridBagAdd(this, sixteenTimes, gbc, 1);
      
//      menuBar = (new EGPPlaybackMenuBar(this)).getMenuBar();
//      menuBar.setEnabled(false);
//      menuBar.setVisible(false);
//      gbc.weightx = 0.1;
//      gbc.weighty = 0.1;
//      UIUtil.jGridBagAdd(this, menuBar, gbc, 1);
      /******************** playback controls ************************/  
      statusText = new StatusLabel();  
      statusText.setToolTipText("Displays protocol status messages.");
      statusText.setText("                    ");
      Dimension d = statusText.getMinimumSize();
      d.width = FixedWidth;
//       leave min height as defined by the font
      statusText.setMinimumSize(d);

      d = statusText.getMaximumSize();
      d.width = FixedWidth;
//       leave max height as defined by the font
      statusText.setMaximumSize(d);

      d = statusText.getPreferredSize();
      d.width = FixedWidth;
//       leave prefd height as defined by the font
      statusText.setPreferredSize(d);

      gbc.weightx = 0.3;
      UIUtil.jGridBagAdd(this, statusText, gbc, GridBagConstraints.RELATIVE);
      
      JPanel lights = new JPanel(new GridLayout(1, 2));
      conn = new JLabel(StatusBar.CONN_STOP_ICON);
      conn.setToolTipText("Displays connect status.");
      conn.setHorizontalAlignment(JLabel.CENTER);
    
      sendingTimer = new Timer((TIMER_INTERVAL  + 150), sta);
      sending = new JLabel(StatusBar.SEND_STOP_ICON);
      sending.setToolTipText("Displays protocol send status.");
      sending.setHorizontalAlignment(JLabel.CENTER);
    
      receivingTimer = new Timer((TIMER_INTERVAL), rta);
      receiving = new JLabel(StatusBar.RECV_STOP_ICON);
      receiving.setToolTipText("Displays protocol receive status.");
      receiving.setHorizontalAlignment(JLabel.CENTER);

      lights.add(conn);
      lights.add(sending);
      lights.add(receiving);
      lights.setBorder(BorderFactory.createCompoundBorder(
      BorderFactory.createLoweredBevelBorder(),
      BorderFactory.createEmptyBorder(1, 1, 1, 1)));
      
      gbc.weightx = 0.0; 
      UIUtil.jGridBagAdd(this, lights, gbc, GridBagConstraints.REMAINDER);
    }

  /** Handle the playback control button clicks. */
  public void actionPerformed(ActionEvent e) {
		String cmd = e.getActionCommand();
		
//		if (cmd.equals(StatusBar.COPY_TO)) {
//			setSessionClipboard();
//		}
		
		if (cmd.equals(StatusBar.CAD)) {
			try {
				this.viewer.sendCtrlAltDel();
			} catch (IOException e1) {
				log.error("StatusBar" + ":" + "actionPerformed"
						+ ":sendCtrlAltDel " + e1.toString());
			}
		} 
		
		if (cmd.equals(StatusBar.CTRLESC)) {
			try {
				this.viewer.sendCtrlEsc();
			} catch (IOException e1) {
				log.error("StatusBar" + ":" + "actionPerformed"
						+ ":sendCtrlEsc " + e1.toString());
			}
		} 
		
		if (cmd.equals(StatusBar.REFRESH)) {
			try {
				this.viewer.refresh();
			} catch (IOException e1) {
				log.error("StatusBar" + ":" + "actionPerformed"
						+ ":refresh " + e1.toString());
			}
		} 
		
		if (cmd.equals(StatusBar.TAB)) {
			tabForward();
		} else {
			tabBackward();
		}

		if (cmd.equals(StatusBar.PAUSE)) {
			paused = paused ? false : true;
			setPause();
		}
		
		if (cmd.equals(StatusBar.LOOP)) {
			loop = loop ? false : true;
			log.debug("StatusBar" + ":" + "actionPerformed"
					+ " " + "call --> setLoopButton()");
			setLoopButton();
		} else {
			if (this.playback) {
				speedActionPerformed(e);
			}
		}
	}
    
	public void setSessionClipboard() {
		if (allowClipboard) {
			if (!block) {
				block = true;
				Clipboard clipboard = Toolkit.getDefaultToolkit()
				.getSystemClipboard();  
				try {
					Transferable content = clipboard.getContents(this);
					if (content != null) {
						String dstData = (String) content
								.getTransferData(DataFlavor.stringFlavor);
						viewer.setCutText(dstData);
						dstData = null;
						content = null;
					}
				} catch (Exception e) {
					log.error("StatusBar" + ":" + "setSessionClipboard"
							+ " " + e.toString());
				}
			    block = false;
			}
		}
	}
	
	public void setClientClipboard(String s) {
		if (!isPlayback() && allowClipboard) {
			if (!block) {
				block = true;
				 Clipboard clipboard = Toolkit.getDefaultToolkit()
					.getSystemClipboard();  
				try {
					StringSelection contents = new StringSelection(s);
					clipboard.setContents(contents, this);
				} catch (Exception e) {
					log.error("StatusBar" + ":" + "setClientClipboard" + " "
							+ e.toString());
				}
				block = false;
			}
		}
		s = null;
	}
  
  
  
  public void tabForward() {
// viewer.processTabEvent();
  }
  
  public void tabBackward() {
//	  viewer.processShiftTabEvent();
  }
  
  public void speedActionPerformed(ActionEvent e) { 
      if (log.isDebugEnabled()) {
        log.debug("enter :" + "StatusBar" + ":" + "speedActionPerformed");
    }
      String choice = group.getSelection().getActionCommand();  
      speedAction(choice);    
  }
  
  public void speedAction(String choice) { 
      if (log.isDebugEnabled()) {
        log.debug("enter :" + "StatusBar" + ":" + "speedAction");
    }
      
      if (choice.equals(StatusBar.HALF_SPEED)) {
    	  sendingTimer.stop();
    	  receivingTimer.stop();
          speed = 0.5;
          viewer.processPlaybackSpeedEvent(StatusBar.HALF_SPEED);
      } else if (choice.equals(StatusBar.NORMAL_SPEED)) {
    	  sendingTimer.stop();
    	  receivingTimer.stop();
          speed = 1;
          viewer.processPlaybackSpeedEvent(StatusBar.NORMAL_SPEED);
      } else if (choice.equals(StatusBar.TWO_TIMES_SPEED)) {
    	  sendingTimer.start();
    	  receivingTimer.start();
          speed = 2;
          viewer.processPlaybackSpeedEvent(StatusBar.TWO_TIMES_SPEED);
      } else if (choice.equals(StatusBar.FOUR_TIMES_SPEED)) {
    	  sendingTimer.start();
    	  receivingTimer.start();
          speed = 4;
          viewer.processPlaybackSpeedEvent(StatusBar.FOUR_TIMES_SPEED);
      } else if (choice.equals(StatusBar.EIGHT_TIMES_SPEED)) {
    	  sendingTimer.start();
    	  receivingTimer.start();
          speed = 8;
          viewer.processPlaybackSpeedEvent(StatusBar.EIGHT_TIMES_SPEED);
      } else if (choice.equals(StatusBar.SIXTEEN_TIMES_SPEED)) {
    	  sendingTimer.start();
    	  receivingTimer.start();
          speed = 16;
          viewer.processPlaybackSpeedEvent(StatusBar.SIXTEEN_TIMES_SPEED);
      }
  }
  
  public void setPlaybackControls(boolean visible) {
	    
	    if (!visible) {
			  cfl = new CBFlavorListener();
			  Clipboard clipboard = Toolkit.getDefaultToolkit()
				.getSystemClipboard();  
			  clipboard.addFlavorListener(cfl);
	    }
	  
		this.playback = visible;
		
		cad.setVisible(!visible);
		ctrlesc.setVisible(!visible);
		refresh.setVisible(!visible);

//		menuBar.setVisible(visible);
		half.setVisible(visible);
		normal.setVisible(visible);
		twoTimes.setVisible(visible);
		fourTimes.setVisible(visible);
		eightTimes.setVisible(visible);
		sixteenTimes.setVisible(visible);
		
		counter.setVisible(visible);
		countTo.setVisible(visible);
		pause.setVisible(visible);
		loopButton.setVisible(visible);
		pbar.setVisible(visible);
		if (!visible) {
			viewer.showWelcomeScreen();
		}
	}
  
  public void setAllowClipboard(boolean allow) {  
	    allowClipboard = allow;
  }
  
  public void setAllowCAD(boolean allow) {
	    allowCAD = allow;
	    if (!allow) {
	    	cad.setVisible(false);
	    	ctrlesc.setVisible(false);
	    }
  }
  
  public void setPlaybackControlsActive(boolean active) {
	  	if (speed > 1) {
	  		sendingTimer.start();
	    	receivingTimer.start();
	  	} else {
	  		sendingTimer.stop();
	    	receivingTimer.stop();
	  	}
//	  	menuBar.setEnabled(active);
		half.setEnabled(active);
		normal.setEnabled(active);
		twoTimes.setEnabled(active);
		fourTimes.setEnabled(active);
		eightTimes.setEnabled(active);
		sixteenTimes.setEnabled(active);
		
		pause.setEnabled(active);
		loopButton.setEnabled(active);
		sliderAction = false;
		enableSpeedControls(active); 
	}
  
  public void setNoControls() {	 
	  sendingTimer.removeActionListener(sta);
      receivingTimer.removeActionListener(rta);
	  setStatusText(StatusBar.CLOSED);
      setConnected(false);  
      setReceiving(false);
      setSending(false);
     
	  cad.setVisible(false);
	  ctrlesc.setVisible(false);
	  refresh.setVisible(false);

//	  menuBar.setVisible(false);
      half.setVisible(false);
      normal.setVisible(false);
      twoTimes.setVisible(false);
      fourTimes.setVisible(false);
      eightTimes.setVisible(false);
      sixteenTimes.setVisible(false);
      
	  counter.setVisible(false);
	  countTo.setVisible(false);
	  pause.setVisible(false);
	  loopButton.setVisible(false);
	  pbar.setVisible(false);
  }
  
  public void stopCounter() {
      countTimer.stop();
      /* Replay ended disable the playback buttons */
      pause.setBorder(BorderFactory.createRaisedBevelBorder());    
      pause.setFocusable(false);
      pause.setEnabled(false);
      /* give the bar one last tweak! */
      counter.setText(DurationFormatUtils.formatPeriod(0, duration, StatusBar.TIME_FORMAT));
      pbar.setValue((int)duration);
      enableSpeedControls(false); 
  }
  
//  /**
// * This method handles keyboard events only. Simulates button click.
// */
//public void togglePauseMode() {
//      if (log.isDebugEnabled()) {
//        log.debug("enter :" + "StatusBar" + ":" + "togglePauseMode");
//      }
//      paused = paused ? false : true;
//      log.debug("StatusBar" + ":" + "togglePauseMode"
//            + " " + paused);
//      setControls();
//  }
  
  public void handleJumpToSelectedEvent() {
	  setPlaybackControlsActive(false); 
	  sliderAction = true;
	  pbar.setEnabled(false);
	  sending.setEnabled(true);
	  receiving.setEnabled(true);
	  countTimer.start();
	  pause.setBorder(BorderFactory.createRaisedBevelBorder());   
	  pauseTimer.stop();
	  pause.setIcon(GREEN_LED_ON);  
	  sendingTimer.addActionListener(sta);
	  receivingTimer.addActionListener(rta);
	  this.setPaused(false);
  }
  
  public void setPause() {
	  viewer.processPauseEvent();
	  setPauseUI();
  }
  
  private void setPauseUI() {
      if (paused) {
          pbar.setEnabled(true);
          pausedValue = pbar.getValue();
          countTimer.stop();
          pause.setBorder(BorderFactory.createLoweredBevelBorder());   
          pauseTimer.start();
          pause.setIcon(RED_LED_ON);  
          enableSpeedControls(false); 
          sendingTimer.removeActionListener(sta);
          receivingTimer.removeActionListener(rta);
          setSending(false);
          setReceiving(false);
      } else {
          /* process the jump to (rewind or fast-forward) */
          pbar.setEnabled(false);
          int val = pbar.getValue();
          if (val != pausedValue) {
              now = val;
              setPlaybackControlsActive(false);
              sliderAction = true;
              viewer.processJumpEvent(val);
          } else {
        	  enableSpeedControls(true); 
          }
	      sending.setEnabled(true);
	      receiving.setEnabled(true);
          countTimer.start();
          pause.setBorder(BorderFactory.createRaisedBevelBorder());   
          pauseTimer.stop();
          pause.setIcon(GREEN_LED_ON);  
          sendingTimer.addActionListener(sta);
          receivingTimer.addActionListener(rta);
      }
  }

  public void setLoopButton() { 
      viewer.processLoopEvent();
      if (loop) {
          loopButton.setBorder(BorderFactory.createLoweredBevelBorder());        
          loopButton.setIcon(GREEN_LED_ON);  

      } else {
          /* process the jump to (rewind or fast-forward) */
    	  loopButton.setBorder(BorderFactory.createRaisedBevelBorder());   
          loopButton.setIcon(RED_LED_ON);  
      }
  }
  
private void enableSpeedControls(boolean bool) {
    half.setFocusable(bool);
    half.setEnabled(bool);
    normal.setFocusable(bool);
    normal.setEnabled(bool);
    twoTimes.setFocusable(bool);
    twoTimes.setEnabled(bool);
    fourTimes.setFocusable(bool);
    fourTimes.setEnabled(bool);
    eightTimes.setFocusable(bool);
    eightTimes.setEnabled(bool);
    sixteenTimes.setFocusable(bool);
    sixteenTimes.setEnabled(bool);
    
//    menuBar.setFocusable(bool);
//    menuBar.setEnabled(bool);
}
  /**
   *
   *
   * @param receiving
   */
  public void setReceiving(boolean receiving) {
	  if (speed < 2 && (!sliderAction)) { 
		  this.receiving.setIcon(receiving ? StatusBar.RECV_ON_ICON : StatusBar.RECV_OFF_ICON);
	  } 
//	  this.setStatusText(StatusBar.RECEIVED);
      if (!started) {
          started = true;
          countTimer.start();
      }
  }

  /**
   *
   *
   * @param sending
   */
  public void setSending(boolean sending) {
	  if (speed < 2 && (!sliderAction)) { 
		  this.sending.setIcon(sending ? StatusBar.SEND_ON_ICON : StatusBar.SEND_OFF_ICON);
	  }
//	  this.setStatusText(StatusBar.SENT);
  }

  /**
   *
   *
   * @param connected
   */
  public void setConnected(boolean connect) {
    if (connect) {
    	this.conn.setIcon(StatusBar.CONN_ON_ICON);
//    	try {
//			viewer.sendCtrlEsc();
//		} catch (IOException e) {
//			log.error("StatusBar" + ":" + "setConnected"
//					+ " " + e.toString());
//		}
  	} else {
    	this.receiving.setIcon(StatusBar.RECV_STOP_ICON);
    	this.sending.setIcon(StatusBar.SEND_STOP_ICON);
    	this.conn.setIcon(StatusBar.CONN_STOP_ICON);
    }
  }

  /**
   *
   *
   * @param text
   */
  public void setStatusText(String text) {
    synchronized (statusText) {
    	statusText.setText(text);
    }
  }

  /**
   *
   *
   * @param text
   */
  public void setHost(String text) {
    host.setText(text);
  }
  
  public String getHost() {
	 return host.getText();
  }

  /**
   *
   *
   * @param text
   * @param port
   */
  public void setHost(String text, int port) {
    host.setText(text + ":" + String.valueOf(port));
  }

  /**
   *
   *
   * @param remoteId
   */
  public void setRemoteId(String remoteId) {
//    rid.setText(remoteId);
  }

  /**
   *
   *
   * @param text
   */
  public void setUser(String text) {
    user.setText(text);
  }
  
  public String getUser() {
 	return user.getText();
  }
  

class CBFlavorListener implements FlavorListener {
		Field field = null;

		public CBFlavorListener() {
			super();
			this.init();
		}
		
		/*
		 * Unfortunately the standard implementation will only inform about
		 * "flavor" changes. This means that we will get a notification when a
		 * different kind of item is copied to the clipboard but not when a
		 * different item of the same kind is copied (for example when you copy
		 * two simple text strings from a text editor, you may be notfied of the
		 * first copy but not of the second copy).
		 * 
		 * The only way to see every copy in java is polling frequently and
		 * comparing the contents to see if something new is on the clipboard.
		 * This may be the most portable way but it is also wasteful and
		 * unnecessary on windows.
		 * 
		 * To implement non-wasteful clipboard monitoring I looked into the
		 * source code of the jdk. In the source code I found the implementation
		 * of the class "sun.awt.windows.WClipboard". And when I read the code I
		 * saw that this implementation receives a notification from the VM for
		 * every copy to the system clipboard. It then filters these
		 * notifications to only notify the listeners when a "flavor" change has
		 * happened!. So we get less information than the OS and the VM are
		 * giving us.
		 * 
		 * The code that checks if a new flavor has been copied to the clipboard
		 * uses a field to store the flavors of the previous contents of the
		 * clipboard. This field is called "currentDataFlavors" (it is a field
		 * of the superclass "sun.awt.datatransfer.SunClipboard"). We only have
		 * to set this field to "null" every time we get notified to see every
		 * copy to the clipboard.
		 * 
		 * With the following code we will receive a notification of every copy:
		 */

		private void init() {
			try {
				field = sun.awt.datatransfer.SunClipboard.class
						.getDeclaredField(StatusBar.CDF);
				field.setAccessible(true);
			} catch (SecurityException ex) {
				field = null;
			} catch (NoSuchFieldException ex) {
				field = null;
			}
		}

		public void flavorsChanged(FlavorEvent e) {
			log.debug("CBFlavorListener" + ":" + "flavorsChanged" + " "
					+ "clipboard changed!");
			try {
				Clipboard clipboard = Toolkit.getDefaultToolkit()
				.getSystemClipboard();
				if (field != null) {
					field.set(clipboard, null);
				}
			} catch (Exception ex) {
				log.error("CBFlavorListener" + ":" + "flavorsChanged"
						+ " " + ex.toString());
			}
			setSessionClipboard();
		}
	}

  
  class DummyButton extends JButton {
	  /**
	 * 
	 */
	private static final long serialVersionUID = 1L;
    
	DummyButton() {
	      super();
	      init();
    }

    void init() {
    	setBorder(BorderFactory.createCompoundBorder(
                BorderFactory.createLoweredBevelBorder(),
                BorderFactory.createEmptyBorder(1, 1, 1, 1)));
        setFocusable(true);
        setEnabled(true);
    }
  }
  
  
  class BarPanel extends JPanel {
	  /**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	BarPanel(GridLayout gl) {
		  super(gl);
		  init();
    }
    
	  BarPanel() {
	      super();
	      init();
    }

    void init() {
    	setBorder(BorderFactory.createLoweredBevelBorder());
    }
  }
  
  class StatusLabel extends JLabel {
/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	StatusLabel(Icon icon) {
		  super(icon);
		  init();
	}
	
	StatusLabel(String text) {
		    super(text);
		    init();
	 }
	
	StatusLabel() {
		  super();
		  init();
	}
	
	void init() {
	  setFont(getFont().deriveFont(10f));
	  setBorder(BorderFactory.createCompoundBorder(
	      BorderFactory.createLoweredBevelBorder(),
	      BorderFactory.createEmptyBorder(1, 1, 1, 1)));
	}
}

   public SshVNCViewer getViewer() {
      return this.viewer;
   }
   
   class TimerAction implements ActionListener {
       public void actionPerformed(ActionEvent e) {
           pause.setIcon((pause.getIcon() == RED_LED_ON) ? RED_LED_OFF : RED_LED_ON);
       }
   }
   
   class SendTimerAction implements ActionListener {
       public void actionPerformed(ActionEvent e) {
           sending.setIcon((sending.getIcon() == SEND_ON_ICON) ? SEND_OFF_ICON : SEND_ON_ICON);
       }
   }
   
   class ReceiveTimerAction implements ActionListener {
       public void actionPerformed(ActionEvent e) {
           receiving.setIcon((receiving.getIcon() == RECV_ON_ICON ) ? RECV_OFF_ICON : RECV_ON_ICON);
       }
   }
   
   class CounterAction implements ActionListener {
       public void actionPerformed(ActionEvent e) {
           if (now > duration) {
               now = 0;
           }
           counter.setText(null);
           counter.setText(DurationFormatUtils.formatPeriod(0, now, StatusBar.TIME_FORMAT));
           pbar.setValue((int)now);
       }
   } 
   
   class SliderAjustmentListener implements ChangeListener {
       public void stateChanged(ChangeEvent e) {
           if (paused) {
               counter.setText(DurationFormatUtils.formatPeriod(0, pbar.getValue(), StatusBar.TIME_FORMAT));
           }
       }
   }
   

    
    public void setDuration(int duration) {
        this.duration = duration;      
        setSliderTicks(duration);
        initTimer(duration);
        countTo.setText("/" + DurationFormatUtils.formatPeriod(0, this.duration, StatusBar.TIME_FORMAT));
    }
    
    /*
     * If it's less then or equal to 10 minutes 
     *      Major ticks = 1 minute
     *      Minor ticks = 10 seconds
     * If it's greater the 10 minutes and less then or equal to an hour 
     *      Major ticks = 10 minutes
     *      Minor ticks = 1  minute
     * If it's more then 1 hour and less then or equal to 24 hours 
     *      Major ticks = 1  hour
     *      Minor ticks = 10 minutes
     * If it's more then 24 hours
     *      Major ticks = 1  day
     *      Minor ticks = 1  hour
     */ 
    private void setSliderTicks(int duration) {
        pbar.setPaintTicks(true);
        if (duration <= 600000) {
            pbar.setMajorTickSpacing(60000);
            pbar.setMinorTickSpacing(10000);
        } else if ((duration > 600000) & (duration <= 3600000)) {
            pbar.setMajorTickSpacing(600000);
            pbar.setMinorTickSpacing(60000);
        } else if ((duration > 3600000) & (duration <= 86400000)) {
            pbar.setMajorTickSpacing(3600000);
            pbar.setMinorTickSpacing(600000);
        } else {
            pbar.setMajorTickSpacing(86400000);
            pbar.setMinorTickSpacing(3600000);
        }
    }

    /**
     * @param duration
     */
    private void initTimer(int duration) {
        /* initialize the counter and progress barr */
        countTimer = new Timer(StatusBar.TIMER_INTERVAL, new CounterAction());
//        countTimer.setInitialDelay(0);
//        countTimer.setCoalesce(false);
        pbar.setMinimum(0);
        pbar.setMaximum(duration);
        now = 0;
        counter.setText(DurationFormatUtils.formatPeriod(0, now, StatusBar.TIME_FORMAT));
        pbar.setValue((int)now);
    }


    public void setNow(long now) {
        this.now = now;
    }


	public void setLoop(boolean loop) {
		this.loop = loop;
	}


	public boolean isLoop() {
		return loop;
	}


	public void lostOwnership(Clipboard clipboard, Transferable contents) {
		log.debug("StatusBar" + ":" + "lostOwnership"
				+ " " + "problem?");
	}


	public boolean isPlayback() {
		return playback;
	}
	
	public boolean isAllowCAD() {
		return allowCAD;
	}


	/**
	 * @return the paused
	 */
	public boolean isPaused() {
		return paused;
	}


	/**
	 * @param paused the paused to set
	 */
	public void setPaused(boolean paused) {
		this.paused = paused;
	}
}
