/*
 *  SSHTools - Java SSH2 API
 *
 *  Copyright (C) 2002 Lee David Painter.
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public License
 *  as published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *
 *  You may also distribute it and/or modify it under the terms of the
 *  Apache style J2SSH Software License. A copy of which should have
 *  been provided with the distribution.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  License document supplied with your distribution for more details.
 *
 */

package com.sshtools.j2ssh.agent;

import java.io.IOException;

import com.sshtools.j2ssh.io.ByteArrayReader;
import com.sshtools.j2ssh.io.ByteArrayWriter;
import com.sshtools.j2ssh.subsystem.SubsystemMessage;

class SshAgentOperationComplete
    extends SubsystemMessage {
  /**  */
  public static final int SSH_AGENT_OPERATION_COMPLETE = 105;
  private byte[] data;

  /**
   * Creates a new SshAgentOperationComplete object.
   */
  public SshAgentOperationComplete() {
    super(SSH_AGENT_OPERATION_COMPLETE);
  }

  /**
   * Creates a new SshAgentOperationComplete object.
   *
   * @param data
   */
  public SshAgentOperationComplete(byte[] data) {
    super(SSH_AGENT_OPERATION_COMPLETE);
    this.data = data;
  }

  /**
   *
   *
   * @return
   */
  public byte[] getData() {
    return data;
  }

  /**
   *
   *
   * @return
   */
  public String getMessageName() {
    return "SSH_AGENT_OPERATION_COMPLETE";
  }

  /**
   *
   *
   * @param baw
   *
   * @throws java.io.IOException
   * @throws com.sshtools.j2ssh.transport.InvalidMessageException DOCUMENT
   *         ME!
   */
  public void constructByteArray(ByteArrayWriter baw) throws java.io.
      IOException,
      com.sshtools.j2ssh.transport.InvalidMessageException {
    try {
      baw.writeBinaryString(data);
    }
    catch (IOException ioe) {
    }
  }

  /**
   *
   *
   * @param bar
   *
   * @throws java.io.IOException
   * @throws com.sshtools.j2ssh.transport.InvalidMessageException DOCUMENT
   *         ME!
   */
  public void constructMessage(ByteArrayReader bar) throws java.io.IOException,
      com.sshtools.j2ssh.transport.InvalidMessageException {
    try {
      data = bar.readBinaryString();
    }
    catch (IOException ioe) {
    }
  }
}
