/*
 *  SSHTools - Java SSH2 API
 *
 *  Copyright (C) 2002 Lee David Painter.
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public License
 *  as published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *
 *  You may also distribute it and/or modify it under the terms of the
 *  Apache style J2SSH Software License. A copy of which should have
 *  been provided with the distribution.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  License document supplied with your distribution for more details.
 *
 */

package com.sshtools.j2ssh.agent;

import java.io.IOException;

import com.sshtools.j2ssh.io.ByteArrayReader;
import com.sshtools.j2ssh.io.ByteArrayWriter;
import com.sshtools.j2ssh.io.UnsignedInteger32;
import com.sshtools.j2ssh.subsystem.SubsystemMessage;
import com.sshtools.j2ssh.transport.InvalidMessageException;

class SshAgentVersionResponse
    extends SubsystemMessage {
  /**  */
  public static final int SSH_AGENT_VERSION_RESPONSE = 103;
  UnsignedInteger32 version;

  /**
   * Creates a new SshAgentVersionResponse object.
   */
  public SshAgentVersionResponse() {
    super(SSH_AGENT_VERSION_RESPONSE);
  }

  /**
   * Creates a new SshAgentVersionResponse object.
   *
   * @param version
   */
  public SshAgentVersionResponse(int version) {
    super(SSH_AGENT_VERSION_RESPONSE);
    this.version = new UnsignedInteger32(version);
  }

  /**
   *
   *
   * @return
   */
  public int getVersion() {
    return version.intValue();
  }

  /**
   *
   *
   * @return
   */
  public String getMessageName() {
    return "SSH_AGENT_VERSION_RESPONSE";
  }

  /**
   *
   *
   * @param baw
   *
   * @throws java.io.IOException
   * @throws com.sshtools.j2ssh.transport.InvalidMessageException DOCUMENT
   *         ME!
   * @throws InvalidMessageException
   */
  public void constructByteArray(ByteArrayWriter baw) throws java.io.
      IOException,
      com.sshtools.j2ssh.transport.InvalidMessageException {
    try {
      baw.writeUINT32(version);
    }
    catch (IOException ex) {
      throw new InvalidMessageException(ex.getMessage());
    }
  }

  /**
   *
   *
   * @param bar
   *
   * @throws java.io.IOException
   * @throws com.sshtools.j2ssh.transport.InvalidMessageException DOCUMENT
   *         ME!
   * @throws InvalidMessageException
   */
  public void constructMessage(ByteArrayReader bar) throws java.io.IOException,
      com.sshtools.j2ssh.transport.InvalidMessageException {
    try {
      version = bar.readUINT32();
    }
    catch (IOException ex) {
      throw new InvalidMessageException(ex.getMessage());
    }
  }
}
