//
//  Copyright (C) 2001 HorizonLive.com, Inc.  All Rights Reserved.
//  Copyright (C) 1999 AT&T Laboratories Cambridge.  All Rights Reserved.
//
//  This is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This software is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this software; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
//  USA.
//
package com.sshtools.sshvnc;

//
// Clipboard frame.
//
import java.awt.BorderLayout;
import java.awt.Dialog;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.Frame;
import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.ClipboardOwner;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.StringSelection;
import java.awt.datatransfer.Transferable;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

import javax.swing.BorderFactory;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.sshtools.common.ui.IconWrapperPanel;
import com.sshtools.common.ui.ResourceIcon;
import com.sshtools.common.ui.StatusBar;

/*
 * 3/14/11 Bob Baske - Modified to enhance clipboard functionality.
 */

class ClipboardFrame extends JDialog implements WindowListener, ActionListener,
		DocumentListener, ClipboardOwner {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	private static Log log = LogFactory.getLog(ClipboardFrame.class);

	JTextArea textArea;

	JButton clearButton, closeButton;

	JButton setButton, getButton;

	String selection = "";

	SshVNCViewer viewer;

	public final static Icon CLIP_ICON = new ResourceIcon(StatusBar.class,
			"/resources/largeclipboard.png");

	//
	// Constructor.
	//

	ClipboardFrame(Frame parent, SshVNCViewer v) {
		super(parent, "VNC Clipboard", false);
		init(v);
	}

	//
	// Constructor.
	//

	ClipboardFrame(Dialog parent, SshVNCViewer v) {
		super(parent, "VNC Clipboard", false);
		init(v);
	}

	//
	// Constructor.
	//

	ClipboardFrame(SshVNCViewer v) {
		super((Frame) null, "VNC Clipboard", false);
		init(v);
	}

	ClipboardFrame() {
		super();
	}

	private void init(SshVNCViewer v) {
		viewer = v;
		initUI();
	}

	private void initUI() {
		JPanel b = new JPanel(new FlowLayout(FlowLayout.RIGHT, 2, 2));
		b.setBorder(BorderFactory.createEmptyBorder(4, 4, 4, 4));
		
		Font f = new Font("Serif,", Font.BOLD , 11);

		getButton = new JButton("Get Client Clipboard");
		getButton.setToolTipText("Click to get contents of Client clipboard.");
		getButton.setFont(f);
		getButton.setMnemonic('g');
		getButton.addActionListener(this);
		b.add(getButton);

		setButton = new JButton("Set Session Clipboard");
		setButton.setToolTipText("Click to set remote session clipboard.");
		setButton.setFont(f);
		setButton.setMnemonic('s');
		setButton.addActionListener(this);
		b.add(setButton);

		clearButton = new JButton("Clear");
		clearButton.setToolTipText("Click to clear all clipboards.");
		clearButton.setFont(f);
		clearButton.setMnemonic('l');
		clearButton.addActionListener(this);
		b.add(clearButton);

		closeButton = new JButton("Close");
		closeButton.setToolTipText("Click to close VNC clipboard.");
		closeButton.setFont(f);
		closeButton.setMnemonic('c');
		closeButton.addActionListener(this);
		closeButton.setDefaultCapable(true);
		b.add(closeButton);

		JPanel t = new JPanel(new BorderLayout());
		textArea = new JTextArea(8, 30);
		textArea.getDocument().addDocumentListener(this);
		t.add(new JScrollPane(textArea), BorderLayout.CENTER);

		IconWrapperPanel w = new IconWrapperPanel(CLIP_ICON, t);
		w.setBorder(BorderFactory.createEmptyBorder(4, 4, 0, 4));

		getContentPane().setLayout(new BorderLayout());
		getContentPane().add(w, BorderLayout.CENTER);
		getContentPane().add(b, BorderLayout.SOUTH);

		pack();
		setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
		addWindowListener(this);
	}

	public void insertUpdate(DocumentEvent e) {
		updateSessionClipboard();
	}

	public void removeUpdate(DocumentEvent e) {
		updateSessionClipboard();
	}

	public void changedUpdate(DocumentEvent e) {
		updateSessionClipboard();
	}

	private void updateSessionClipboard() {
		if (selection != null && !selection.equals(textArea.getText())) {
			log.debug("ClipboardFrame" + ":" + "updateSessionClipboard" + " "
					+ textArea.getText());
			selection = textArea.getText();
			viewer.setCutText(selection);
		}
	}

	public void setCutText(String text) {
		selection = text;
		textArea.setText(text);
		if (isVisible()) {
			textArea.requestFocus();
			textArea.selectAll();
			try {
				Clipboard clipboard = Toolkit.getDefaultToolkit()
						.getSystemClipboard();
				StringSelection contents = new StringSelection(text);

				clipboard.setContents(contents, this);
			} catch (Exception e) {
				log.error("ClipboardFrame" + ":" + "setCutText" + " "
						+ e.toString());
			}
		}
	}

	private void getClientClipboard() {
		try {
			Clipboard clipboard = Toolkit.getDefaultToolkit()
					.getSystemClipboard();
			Transferable content = clipboard.getContents(this);
			if (content != null) {
				String dstData = (String) content
						.getTransferData(DataFlavor.stringFlavor);
				textArea.setText(dstData);
				textArea.requestFocus();
				textArea.selectAll();
				viewer.setCutText(dstData);
			}
		} catch (Exception e) {
			log.error("ClipboardFrame" + ":" + "setCutText" + " "
					+ e.toString());
		}
	}

	public JButton getDefaultButton() {
		return closeButton;
	}

	public void windowClosing(WindowEvent evt) {
		setVisible(false);
	}

	public void windowDeactivated(WindowEvent evt) {
	}

	public void windowActivated(WindowEvent evt) {
	}

	public void windowOpened(WindowEvent evt) {
	}

	public void windowClosed(WindowEvent evt) {
	}

	public void windowIconified(WindowEvent evt) {
	}

	public void windowDeiconified(WindowEvent evt) {
	}

	//
	// Respond to button presses
	//

	public void actionPerformed(ActionEvent evt) {
		if (evt.getSource() == clearButton) {
			textArea.setText("");
			viewer.setCutText("");
			try {
				Clipboard clipboard = Toolkit.getDefaultToolkit()
						.getSystemClipboard();
				StringSelection contents = new StringSelection("");

				clipboard.setContents(contents, this);
			} catch (Exception e) {
				log.error("ClipboardFrame" + ":" + "actionPerformed:clear"
						+ " " + e.toString());
			}
		} else if (evt.getSource() == closeButton) {
			setVisible(false);
		} else if (evt.getSource() == setButton) {
			textArea.requestFocus();
			textArea.selectAll();
			updateSessionClipboard();
		} else if (evt.getSource() == getButton) {
			textArea.setText("");
			getClientClipboard();
		}
	}

	// A simple test program
	public static void main(String[] args) {
		ClipboardFrame cf = new ClipboardFrame();
		cf.setSize(350, 250);
		cf.setVisible(true);
		cf.initUI();
	}

	public void lostOwnership(Clipboard clipboard, Transferable contents) {
		// TODO Auto-generated method stub

	}
}