//
//  Copyright (C) 2001,2002 HorizonLive.com, Inc.  All Rights Reserved.
//  Copyright (C) 2002 Constantin Kaplinsky.  All Rights Reserved.
//  Copyright (C) 1999 AT&T Laboratories Cambridge.  All Rights Reserved.
//
//  This is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This software is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this software; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
//  USA.
//
package com.sshtools.sshvnc;

//
// VncViewer.java - the VNC viewer applet.  This class mainly just sets up the
// user interface, leaving it to the VncCanvas to do the actual rendering of
// a VNC desktop.
//

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Image;
import java.awt.Label;
import java.awt.Point;
import java.awt.Window;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.io.StringWriter;

import javax.swing.Icon;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableColumn;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.edmz.common.model.EventDataModel;
import com.sshtools.common.ui.ResourceIcon;
import com.sshtools.common.ui.SshToolsApplicationPanel;
import com.sshtools.common.ui.StatusBar;
import com.sshtools.common.ui.UIUtil;

public class SshVNCViewer
    extends JPanel {
  private static Log log = LogFactory.getLog(SshVNCViewer.class);
    
//  public final static Icon LOGO_ICON = new ResourceIcon(SshVNCViewer.class,
//  "/resources/e-DMZ_logo_RGB.jpg");
  
  public final static Icon LOGO_ICON = new ResourceIcon(SshVNCViewer.class,
  "/resources/logo_EGP.jpg");
  
  private StatusBar statusBar;
  private SshVNCPanel vncPanel;

  private String[] mainArgs;
  private RfbProto rfb;

  private JScrollPane desktopScrollPane;
  private GridBagLayout gridbag;
  private VncCanvas vc;
  private ClipboardFrame clipboard;
  private boolean protocolInitialised;

  // Control session recording.
  private Object recordingSync;
  private String sessionFileName;
  private boolean recordingActive;
  private boolean recordingStatusChanged;
  private String cursorUpdatesDef;
  private String eightBitColorsDef;

  // Variables read from parameter values.
  private String socketFactory;
  private boolean showOfflineDesktop;
  private int deferScreenUpdates;
  private int deferCursorUpdates;
  private int deferUpdateRequests;
  private SshToolsApplicationPanel application;
  private SshVNCOptions options;
  private SshVNCOptions oldOptions;
  private AuthenticationFrame authFrame;
  private boolean frameResizeable;
  private boolean ctrlIsEnter = false;
  
  /*
   * 3/14/11 Bob Baske - for event handling
   */
  private EventDataModel edm = null;
  
  private JTable eventTable = null;
  
  private JScrollPane eventScrollPane = null;
  
  JDialog eventDialog = null;

  /**
 * @param ctrlIsEnter the ctrlIsEnter to set
 */
public void setCtrlIsEnter(boolean ctrlIsEnter) {
	this.ctrlIsEnter = ctrlIsEnter;
}

/**
 * @return the ctrlIsEnter
 */
public boolean isCtrlIsEnter() {
	return ctrlIsEnter;
}

public SshVNCViewer() {
    super();
    log.debug("SshVNCViewer" + ":" + "SshVNCViewer");
    frameResizeable = true;
    showWelcomeScreen();

  }

  public boolean init(InputStream in, OutputStream out,
                      SshToolsApplicationPanel application,
                      SshVNCOptions options) throws Exception {
   
    this.options = options;
    this.application = application;

    recordingSync = new Object();

//    Window w = (Window) SwingUtilities.getAncestorOfClass(Window.class, this);
//    if (w instanceof JFrame) {
//      clipboard = new ClipboardFrame( (JFrame) w, this);
//    }
//    else if (w instanceof JDialog) {
//      clipboard = new ClipboardFrame( (JDialog) w, this);
//    }
//    else {
//      clipboard = new ClipboardFrame(this);
//    }

    sessionFileName = null;
    recordingActive = false;
    recordingStatusChanged = false;
    cursorUpdatesDef = null;
    eightBitColorsDef = null;

    this.application = application;

    gridbag = new GridBagLayout();
    removeAll();
    setLayout(gridbag);

    GridBagConstraints gbc = new GridBagConstraints();
    gbc.gridwidth = GridBagConstraints.REMAINDER;
    gbc.anchor = GridBagConstraints.NORTHWEST;

    try {
      if (!connectAndAuthenticate(in, out)) {
        application.getStatusBar().setStatusText(
            "Failed to authenticate with VNC server");
        closeConnection();
        return false;
      }
      application.getStatusBar().setStatusText(
          "Initializing VNC protocol");
      doProtocolInitialisation();
      
      /*
       * 3/14/11 Bob Baske - Load up the events.
       */
      if (application.getStatusBar().isPlayback()) {
    	  application.getStatusBar().setStatusText(
    	  	"Initializing Events");
    	  doEventInitialisation();
      }
      
      
      vc = new VncCanvas(this);

      // Disable the local cursor if required
      if (options.isRequestCursorUpdates() && !options.isIgnoreCursorUpdates()) {
        try {
//                    Image img = new ResourceIcon(
//                        "/com/sshtools/sshvnc/dotcursor.gif").getImage();
          Image img = new BufferedImage(1, 1, BufferedImage.TYPE_4BYTE_ABGR);
          if (img != null) {
            Cursor c = getToolkit().createCustomCursor(img,
                new Point(0, 0), "Dot");
            vc.setCursor(c);
          }
        }
        catch (Throwable t) {
          t.printStackTrace();
        }
      }

      gbc.weightx = 1.0;
      gbc.weighty = 1.0;
      gridbag.setConstraints(vc, gbc);

//            Panel canvasPanel = new Panel();
//            canvasPanel.setLayout(new FlowLayout(FlowLayout.LEFT, 0, 0));
//            canvasPanel.add(vc);
      desktopScrollPane = new JScrollPane(vc);
      desktopScrollPane.setBorder(null);
      gbc.fill = GridBagConstraints.BOTH;
      gridbag.setConstraints(desktopScrollPane, gbc);
//            desktopScrollPane.add(vc);
      add(desktopScrollPane);
      validate();
      vc.resizeDesktopFrame();

      getVncPanel().getApplet().isConnected();
      
      application.getStatusBar().setStatusText("Connected");
      application.getStatusBar().setConnected(true);
      vc.processNormalProtocol();

      return true;
    }
    catch (Exception e) {
      closeConnection();
      throw e;
    }
  }

  public void showWelcomeScreen() {
    synchronized (getTreeLock()) {
      removeAll();
      setLayout(new BorderLayout());
      JPanel p = new JPanel(new BorderLayout());
//      p.setBackground(Color.black);
      p.setBackground(Color.LIGHT_GRAY);
//      p.setForeground(Color.white);
      JLabel welcomeLabel = new JLabel(LOGO_ICON, JLabel.CENTER);
//      JLabel welcomeLabel = new JLabel("Starting session...", JLabel.CENTER);
      welcomeLabel.setForeground(Color.white);
      welcomeLabel.setFont(welcomeLabel.getFont().deriveFont(72f).
                           deriveFont(Font.BOLD + Font.ITALIC));
      p.add(welcomeLabel, BorderLayout.CENTER);
      add(p, BorderLayout.CENTER);
      validate();
    }
  }
  
  public boolean isFrameResizeable() {
    return frameResizeable;
  }

  public void setFrameResizeable(boolean frameResizeable) {
    this.frameResizeable = frameResizeable;
  }

  public void setClipboardVisible(boolean visible) {
    if (clipboard != null) {
      clipboard.setVisible(visible);
    }
  }

  public int getScreenSizePolicy() {
    return options.getScreenSizePolicy();
  }

  public boolean isClipboardVisible() {
    return clipboard != null && clipboard.isVisible();
  }

  public void closeConnection() {
    protocolInitialised = false;
    if (vc != null) {
      vc.enableInput(false);
      vc.setCursor(Cursor.getDefaultCursor());
    }
    if (rfb != null && !rfb.closed()) {
      rfb.close();
    }
    System.out.println("Disconnect");

    if (rfb != null && !rfb.closed()) {
      rfb.close();
    }
    if (clipboard != null) {
      clipboard.dispose();
    }
    if (recordingActive) {
      setRecordingStatus(null);
      try {
        stopRecording();
      }
      catch (IOException ioe) {
        System.err.println("Could not stop recording sesion.");
        ioe.printStackTrace();
      }
    }
    showWelcomeScreen();
  }

  public String getSocketFactory() {
    return socketFactory;
  }

  public RfbProto getRFB() {
    return rfb;
  }

  public SshVNCOptions getOptions() {
    return options;
  }

  public int getDeferCursorUpdates() {
    return deferCursorUpdates;
  }

  public int getDeferUpdateRequests() {
    return deferUpdateRequests;
  }

  public int getDeferScreenUpdates() {
    return deferUpdateRequests;
  }

  public ClipboardFrame getClipboard() {
    return clipboard;
  }

  public VncCanvas getVNCCanvas() {
    return vc;
  }

  public JFrame getFrame() {
    return (JFrame) SwingUtilities.getAncestorOfClass(JFrame.class, this);
  }
  
  public JFrame getAppletFrame() {
	    return (JFrame) SwingUtilities.getAncestorOfClass(JFrame.class, (this.getVncPanel().getApplet()));
	  }

  public JScrollPane getScroller() {
    return desktopScrollPane;
  }

  //
  // Connect to the RFB server and authenticate the user.
  //

  private boolean connectAndAuthenticate(InputStream in, OutputStream out) throws
      Exception {
    protocolInitialised = false;

    if (authFrame == null) {
      Window w = (Window) SwingUtilities.getAncestorOfClass(Window.class, this);
      if (w instanceof JFrame) {
        authFrame = new AuthenticationFrame( (JFrame) w);
      }
      else if (w instanceof JDialog) {
        authFrame = new AuthenticationFrame( (JDialog) w);
      }
      else {
        authFrame = new AuthenticationFrame();
      }
      UIUtil.positionComponent(SwingConstants.CENTER, authFrame);
    }

    rfb = new RfbProto(in, out, this);
    rfb.readVersionMsg();
    System.out.println("RFB server supports protocol version " +
                       rfb.serverMajor + "." + rfb.serverMinor);

    rfb.writeVersionMsg();

    int authScheme = rfb.readAuthScheme();
    System.out.println("Got auth scheme " + authScheme);

    while (true) {
      switch (authScheme) {

        case RfbProto.NoAuth:
          return true;
        case RfbProto.VncAuth:
          String pw = "";

          String encPw = options.getEncryptedPassword();
          if (!encPw.equals("")) {
            // Encrypted password is hexascii-encoded. Decode.
            byte[] c = HexASCII.ToByteArray(encPw);

            // Decrypt the password.
            byte[] key = {
                23, 82, 107, 6, 35, 78, 88, 7};
            DesCipher des = new DesCipher(key);
            des.decrypt(c, 0, c, 0);
            pw = new String(c);
          }
          else {
            // No supplied password, use GUI dialog
            char[] c = authFrame.doAuthentication();
            if (c == null) {
              return false;
            }
            pw = new String(c);
          }
          ;

          byte[] challenge = new byte[16];
          rfb.is.readFully(challenge);

          if (pw.length() > 8) {
            pw = pw.substring(0, 8); // Truncate to 8 chars

            // vncEncryptBytes in the UNIX libvncauth truncates password
            // after the first zero byte. We do to.
          }
          int firstZero = pw.indexOf(0);
          if (firstZero != -1) {
            pw = pw.substring(0, firstZero);

          }
          byte[] key = {
              0, 0, 0, 0, 0, 0, 0, 0};
          System.arraycopy(pw.getBytes(), 0, key, 0, pw.length());

          DesCipher des = new DesCipher(key);

          des.encrypt(challenge, 0, challenge, 0);
          des.encrypt(challenge, 8, challenge, 8);

          rfb.os.write(challenge);

          int authResult = rfb.is.readInt();

          switch (authResult) {
            case RfbProto.VncAuthOK:
              return true;
            case RfbProto.VncAuthFailed:
              break;
            case RfbProto.VncAuthTooMany:
              throw new Exception(
                  "VNC authentication failed - too many tries");
            default:
              throw new Exception(
                  "Unknown VNC authentication result " +
                  authResult);
          }
          break;

        default:
          throw new Exception("Unknown VNC authentication scheme " +
                              authScheme);
      }
    }
  }

  public boolean isProtocolInitialised() {
    return protocolInitialised;
  }

  //
  // Do the rest of the protocol initialisation.
  //

  void doProtocolInitialisation() throws IOException {

    rfb.writeClientInit();
    protocolInitialised = true;

    rfb.readServerInit();
    
    System.out.println("Desktop name is " + rfb.desktopName);
    System.out.println("Desktop size is " + rfb.framebufferWidth + " x " +
                       rfb.framebufferHeight);

    setEncodings();
  }
  
  /*
   * 3/14/11 Bob Baske 
   * An attempt at the event list crawler which gets called from
   * VncCanvas --> RfbProto.Sync:. We want to try to indicate somehow
   * the event closest to the sync ms. We'd need a custom indicator since 
   * 'selecting' the row would cause it to jump to that place in the playback
   * creating a feedback loop. 
   */
  void selectEvent(long ms) throws IOException {
	  log.error("SshVNCViewer" + ":" + "selectEvent"
			+ " " + ms);
	  
	  int row = edm.highLightRow(ms);
	  
	  TableCellEditor tce = eventTable.getCellEditor(row, 1);
	  	  
//	  eventTable.clearSelection();
	  
	  if (ms != -1) {
		  row = edm.highLightRow(ms);
		  ListSelectionModel selectionModel = eventTable.getSelectionModel();
		  selectionModel.setSelectionInterval(row, row);
	  }
  }
  /*
   * 3/14/11 Bob Baske - Load up the events.
   */
  
  void doEventInitialisation() throws IOException {
	  if (log.isDebugEnabled()) {
		log.debug("enter :" + "SshVNCViewer" + ":" + "doEventInitialisation");
	  }
//	  EventDataModel edm = new EventDataModel();
	  edm = new EventDataModel();
	   if (rfb.readEventMessages(edm) > 0) {
		   buildEventTable(edm);
	   }
  }
  
  void buildEventTable(final EventDataModel edm) throws IOException {
	  if (log.isDebugEnabled()) {
		log.debug("enter :" + "SshVNCViewer" + ":" + "doEventInitialisation");
	  }
	  
//	  final JTable jt = new JTable(edm);
	  eventTable = new JTable(edm);
	  
	  // do some init on the table
	  JTableHeader th = eventTable.getTableHeader();
	  th.setFont(new Font("Tahoma", Font.BOLD, 11));
	  
	  TableColumn column = null;
		for (int i = 0; i < 2; i++) {
		    column = eventTable.getColumnModel().getColumn(i);
		    column.setPreferredWidth(edm.getColumnSize(i));
		}
	  
	  eventScrollPane = new JScrollPane(eventTable);
	  
	  eventTable.getSelectionModel().addListSelectionListener(
	        new ListSelectionListener() {
	            public void valueChanged(ListSelectionEvent event) {
	            	int selectedRow = eventTable.getSelectedRow();
	                int ts = (edm.getEventDataAt(selectedRow).getMs());
	                 
	                try {
	                	if (getStatusBar().isPaused()) {
	                		processPauseEvent();
	                		getStatusBar().handleJumpToSelectedEvent();
	                	}
	                	rfb.writeJumpRequest(ts);
					} catch (IOException e) {
						StringWriter sw = new StringWriter();
						PrintWriter pw = new PrintWriter(sw);
						e.printStackTrace(pw);
						log.error("SshVNCViewer" + ":" + "showEventTable"
								+ "stack trace... " + sw.toString());
					}
	            }
	        }
		);
	  
	  eventDialog = new JDialog(this.getFrame());
	  eventDialog.add(eventScrollPane);
	  eventDialog.pack();
	  eventDialog.setLocationRelativeTo(this);
	  eventDialog.setAlwaysOnTop(true);
	  eventDialog.setTitle(vncPanel.getEventDialogTitle());  
	  
	  BasicWindowMonitor bwm = new BasicWindowMonitor();
	  
	  if (this.getAppletFrame() == null)
		  log.error("SshVNCViewer" + ":" + "buildEventTable"
			+ " " + "frame is null");
	  else
		  log.error("SshVNCViewer" + ":" + "buildEventTable"
					+ " " + "frame is not null!!!");
	  
	  log.error("SshVNCViewer" + ":" + "buildEventTable"
				+ " " + "created monitor");
	  (this.getAppletFrame()).addWindowListener(bwm);
	  log.error("SshVNCViewer" + ":" + "buildEventTable"
				+ " " + "added listener");
	  
	  showEventDialog();
  }
  
  public void showEventDialog() {
	  if (!eventDialog.isVisible()) {
		  eventDialog.setVisible(true);
//	  	  eventDialog.setLocationRelativeTo(this);
	  }
  }

  //
  // Send current encoding list to the RFB server.
  //

  void setEncodings() {
    try {
      if (rfb != null && rfb.inNormalProtocol) {
        rfb.writeSetEncodings(options.getEncodings(),
                              options.getNumberOfEncodings());
        if (vc != null) {
          vc.softCursorFree();
        }
      }
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  }

  //
  // setCutText() - send the given cut text to the RFB server.
  //

  public void setCutText(String text) {
		if (!this.getStatusBar().isPlayback()) {
			try {
				if (rfb != null && rfb.inNormalProtocol) {
					rfb.writeClientCutText(text);
				}
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		text = null;
	}

  //
  // Order change in session recording status. To stop recording, pass
  // null in place of the fname argument.
  //

  void setRecordingStatus(String fname) {
    synchronized (recordingSync) {
      sessionFileName = fname;
      recordingStatusChanged = true;
    }
  }

  public String getRecordingStatus() {
    return sessionFileName;
  }

  //
  // Start or stop session recording. Returns true if this method call
  // causes recording of a new session.
  //

  boolean checkRecordingStatus() throws IOException {
    synchronized (recordingSync) {
      if (recordingStatusChanged) {
        recordingStatusChanged = false;
        if (sessionFileName != null) {
          startRecording();
          return true;
        }
        else {
          stopRecording();
        }
      }
    }
    return false;
  }

  //
  // Start session recording.
  //

  protected void startRecording() throws IOException {
    synchronized (recordingSync) {
      if (!recordingActive) {
        try {
          oldOptions = (SshVNCOptions) options.clone();
        }
        catch (Throwable t) {

        }
        options.setEightBitColors(false);
        options.setRequestCursorUpdates(false);
        options.setEncodings();
        setEncodings();
      }
      else {
        rfb.closeSession();
      }

      System.out.println("Recording the session in " + sessionFileName);
      rfb.startSession(sessionFileName);
      recordingActive = true;
    }
  }

  //
  // Stop session recording.
  //

  protected void stopRecording() throws IOException {
    synchronized (recordingSync) {
      if (recordingActive) {
        options = oldOptions;
        setEncodings();
        rfb.closeSession();
        System.out.println("Session recording stopped.");
      }
      sessionFileName = null;
      recordingActive = false;
    }
  }
  
  /*
   * 3/14/11 Bob Baske - Handle the pause event
   */
  public void processPauseEvent() {
      if (log.isDebugEnabled()) {
        log.debug("enter :" + "SshVNCViewer" + ":" + "processPauseEvent");
    }
      synchronized (rfb) {
          try {
            rfb.writeKeyEvent(new KeyEvent(this, KeyEvent.KEY_PRESSED, System
                  .currentTimeMillis(), 0, 80, 'p'));
            rfb.writeKeyEvent(new KeyEvent(this, KeyEvent.KEY_RELEASED, System
                  .currentTimeMillis(), 0, 80, 'p'));
          }
          catch (Exception e) {
            e.printStackTrace();
          }
          
          rfb.notify();
        }
    }
  
  //
  // Handle events.
  //
  public void processLoopEvent() {
      if (log.isDebugEnabled()) {
        log.debug("enter :" + "SshVNCViewer" + ":" + "processLoopEvent");
    }
      synchronized (rfb) {
          try {
            rfb.writeKeyEvent(new KeyEvent(this, KeyEvent.KEY_PRESSED, System
                  .currentTimeMillis(), 0, 76, 'l'));
            rfb.writeKeyEvent(new KeyEvent(this, KeyEvent.KEY_RELEASED, System
                  .currentTimeMillis(), 0, 76, 'l'));
          }
          catch (Exception e) {
            e.printStackTrace();
          }
          
          rfb.notify();
        }
    }
  
  //
  // Handle events.
  //
  public void processJumpEvent(int millis) {
      if (log.isDebugEnabled()) {
        log.debug("enter :" + "SshVNCViewer" + ":" + "processJumpEvent");
      }
      log.debug("SshVNCViewer" + ":" + "processJumpEvent"
            + " Jump to ms: " + millis);
      synchronized (rfb) {
          try {
              rfb.writeJumpRequest(millis);
          }
          catch (Exception e) {
            e.printStackTrace();
          }
          
          rfb.notify();
        }
    }
  /*
   * 3/14/11 Bob Baske - For playback speed control.
   */
  public void processPlaybackSpeedEvent(String speed) {
      if (log.isDebugEnabled()) {
        log.debug("enter :" + "SshVNCViewer" + ":"
                + "processPlaybackSpeedEvent");
    }
      synchronized (rfb) {
          try {
            if (speed.equals(StatusBar.HALF_SPEED)) {
                rfb.writeKeyEvent(new KeyEvent(this, KeyEvent.KEY_PRESSED, System
                        .currentTimeMillis(), 0, 53, '5'));
                rfb.writeKeyEvent(new KeyEvent(this, KeyEvent.KEY_RELEASED, System
                        .currentTimeMillis(), 0, 53, '5'));                
            } else if (speed.equals(StatusBar.NORMAL_SPEED)) {
                rfb.writeKeyEvent(new KeyEvent(this, KeyEvent.KEY_PRESSED, System
                        .currentTimeMillis(), 0, 49, '1'));
                rfb.writeKeyEvent(new KeyEvent(this, KeyEvent.KEY_RELEASED, System
                        .currentTimeMillis(), 0, 49, '1'));           
            } else if (speed.equals(StatusBar.TWO_TIMES_SPEED)) {
                rfb.writeKeyEvent(new KeyEvent(this, KeyEvent.KEY_PRESSED, System
                        .currentTimeMillis(), 0, 50, '2'));
                rfb.writeKeyEvent(new KeyEvent(this, KeyEvent.KEY_RELEASED, System
                        .currentTimeMillis(), 0, 50, '2'));           
            } else if (speed.equals(StatusBar.FOUR_TIMES_SPEED)) {
                rfb.writeKeyEvent(new KeyEvent(this, KeyEvent.KEY_PRESSED, System
                        .currentTimeMillis(), 0, 52, '4'));
                rfb.writeKeyEvent(new KeyEvent(this, KeyEvent.KEY_RELEASED, System
                        .currentTimeMillis(), 0, 52, '4'));           
            } else if (speed.equals(StatusBar.EIGHT_TIMES_SPEED)) {
                rfb.writeKeyEvent(new KeyEvent(this, KeyEvent.KEY_PRESSED, System
                        .currentTimeMillis(), 0, 56, '8'));
                rfb.writeKeyEvent(new KeyEvent(this, KeyEvent.KEY_RELEASED, System
                        .currentTimeMillis(), 0, 56, '8'));           
            } else if (speed.equals(StatusBar.SIXTEEN_TIMES_SPEED)) {
                rfb.writeKeyEvent(new KeyEvent(this, KeyEvent.KEY_PRESSED, System
                        .currentTimeMillis(), 0, 57, '9'));
                rfb.writeKeyEvent(new KeyEvent(this, KeyEvent.KEY_RELEASED, System
                        .currentTimeMillis(), 0, 57, '9'));           
            } 
          } catch (Exception e) {
            e.printStackTrace();
          }
          
          rfb.notify();
        }
    }
  
  public void refresh() throws IOException {
      if (log.isDebugEnabled()) {
        log.debug("enter :" + "SshVNCViewer" + ":" + "refresh calling writeFramebufferUpdateRequest");
      }
      rfb.writeFramebufferUpdateRequest(0, 0, rfb.framebufferWidth,
                                      rfb.framebufferHeight, false);
  }

  public void sendCtrlAltDel() throws IOException {
	if (this.getStatusBar().isAllowCAD()) {
	    final int modifiers = InputEvent.CTRL_MASK | InputEvent.ALT_MASK;
	    KeyEvent ctrlAltDelEvent =
	        new KeyEvent(this, KeyEvent.KEY_PRESSED, 0, modifiers,
	                     KeyEvent.VK_DELETE, KeyEvent.CHAR_UNDEFINED);
	    getRFB().writeKeyEvent(ctrlAltDelEvent);
	    ctrlAltDelEvent = new KeyEvent(this, KeyEvent.KEY_RELEASED, 0,
	                                   modifiers, KeyEvent.VK_DELETE,
	                                   KeyEvent.CHAR_UNDEFINED);
	    getRFB().writeKeyEvent(ctrlAltDelEvent);
	}
  }
  
  /*
   * 3/14/11 Bob Baske - Ctrl Esc
   */
  public void sendCtrlEsc() throws IOException {	  
	    final int modifiers = 0;
	  	KeyEvent ctrlEscEvent = new KeyEvent(this, KeyEvent.KEY_PRESSED, 0,
	  			modifiers, KeyEvent.VK_WINDOWS, KeyEvent.CHAR_UNDEFINED);

		getRFB().writeKeyEvent(ctrlEscEvent);
	  	ctrlEscEvent = new KeyEvent(this, KeyEvent.KEY_RELEASED, 0,
	  			modifiers, KeyEvent.VK_WINDOWS, KeyEvent.CHAR_UNDEFINED);

		getRFB().writeKeyEvent(ctrlEscEvent);
	}

  //
  // moveFocusToDesktop() - move keyboard focus either to the
  // VncCanvas or to the AuthPanel.
  //

  void moveFocusToDesktop() {
    if (vc != null && isAncestorOf(vc)) {
      vc.requestFocus();
    }
  }

  synchronized public void fatalError(String str, Exception e) {
    if (rfb != null) {
      if (rfb.closed()) {
        return;
      }
      rfb.close();
    }
    application.showErrorMessage(this, str, "VNC Error", e);
  }

  //
  // Show message text and optionally "Relogin" and "Close" buttons.
  //

  void showMessage(String msg) {
    removeAll();

    Label errLabel = new Label(msg, Label.CENTER);
    errLabel.setFont(new Font("Helvetica", Font.PLAIN, 12));
    setLayout(new FlowLayout(FlowLayout.LEFT, 30, 30));
    add(errLabel);
    validate();
  }

  class AuthenticationDialog
      extends JDialog {

  }

public VncCanvas getVc() {
    return this.vc;
}

public StatusBar getStatusBar() {
    return this.statusBar;
}

public void setStatusBar(StatusBar statusBar) {
    this.statusBar = statusBar;
}

/**
 * @return the vncPanel
 */
public SshVNCPanel getVncPanel() {
	return vncPanel;
}

/**
 * @param vncPanel the vncPanel to set
 */
public void setVncPanel(SshVNCPanel vncPanel) {
	this.vncPanel = vncPanel;
}

class BasicWindowMonitor extends WindowAdapter {
	
	boolean dialogWasOpen = false;
	
	public void windowIconified(WindowEvent e) {
		if (eventDialog.isVisible()) {
			dialogWasOpen = true;
		}
		eventDialog.setVisible(false);
		System.out.println("minimized");
	}

	public void windowDeIconified(WindowEvent e) {
		if (dialogWasOpen) {
			eventDialog.setVisible(true);
		}
		System.out.println("maximized");
	}

	public void windowActivated(WindowEvent e) {
		if (dialogWasOpen) {
			eventDialog.setVisible(true);
		}
		System.out.println("activated");
	}
}


}
